/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003  Ciaran Anscomb
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef GP32
#include "gp32/gptypes.h"
#else
#include <SDL/SDL_types.h>
#endif
#include "filer.h"

char *filer_error = "";

#ifdef GP32
#include "gp32/filer.c"
#else
#include "sdl/filer.c"
#endif

int filer_load_file(char *filename, void *buf, int size) {
	int count;
	FILER_FD fd;
	if (filer_open(filename, FILER_READ, &fd) == -1)
		return -1;
	count = filer_read(fd, buf, size);
	filer_close(fd);
	return count;
}

int filer_write_byte(FILER_FD fd, Uint8 octet) {
	return filer_write(fd, &octet, 1);
}
int filer_write_word16(FILER_FD fd, Uint16 word16) {
	if (filer_write_byte(fd, word16 >> 8) == -1)
		return -1;
	return filer_write_byte(fd, word16 & 0xff);
}
int filer_write_word32(FILER_FD fd, Uint32 word32) {
	if (filer_write_word16(fd, word32 >> 16) == -1)
		return -1;
	return filer_write_word16(fd, word32 & 0xffff);
}

int filer_read_byte(FILER_FD fd, Uint8 *dest) {
	return filer_read(fd, dest, 1);
}
int filer_read_word16(FILER_FD fd, Uint16 *dest) {
	Uint8 octet;
	int ret;
	if (filer_read(fd, &octet, 1) == -1)
		return -1;
	*dest = octet << 8;
	if ((ret = filer_read(fd, &octet, 1)) == -1)
		return -1;
	*dest |= octet;
	return ret;
}
int filer_read_word32(FILER_FD fd, Uint32 *dest) {
	Uint16 word16;
	int ret;
	if (filer_read_word16(fd, &word16) == -1)
		return -1;
	*dest = word16 << 16;
	if ((ret = filer_read_word16(fd, &word16)) == -1)
		return -1;
	*dest |= word16;
	return ret;
}
