/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003  Ciaran Anscomb
 *
 *  See LICENSE for redistribution conditions. */

#ifndef __FILER_H__
#define __FILER_H__

#ifdef GP32
#include <gpstdio.h>
#include "gp32/gptypes.h"
#else
#include <SDL/SDL_types.h>
#endif

#define FILER_READ 0x01
#define FILER_WRITE 0x02

#ifdef GP32
typedef F_HANDLE FILER_FD;
typedef struct {
	GPFILEATTR attr;
	char name[16];
} FILER_DIRENT;

extern FILER_DIRENT *filer_directory;
extern Uint32 filer_dir_count;
#else
typedef int FILER_FD;
#endif

extern char *filer_error;

void filer_init(void);
int filer_open(char *filename, int mode, FILER_FD *fd);
int filer_read(FILER_FD fd, void *buffer, Uint32 size);
int filer_write(FILER_FD fd, void *buffer, Uint32 size);
void filer_close(FILER_FD fd);
int filer_load_file(char *filename, void *buf, int size);
int filer_write_byte(FILER_FD fd, Uint8 octet);
int filer_write_word16(FILER_FD fd, Uint16 word16);
int filer_write_word32(FILER_FD fd, Uint32 word32);
int filer_read_byte(FILER_FD fd, Uint8 *dest);
int filer_read_word16(FILER_FD fd, Uint16 *dest);
int filer_read_word32(FILER_FD fd, Uint32 *dest);
#ifdef GP32
void filer_readdir(char *path);
#endif

#endif  /* __FILER_H__ */
