/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003  Ciaran Anscomb
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "gpdef.h"
#include "gpstdlib.h"
#include "gpgraphic.h"
#include "gpstdio.h"
#include "gpfont.h"
#include "../video.h"
#include "../joystick.h"

#define KEYBOARD_OFFSET (64*240)

extern PIXEL kbd_bin[2][5060];
extern PIXEL cmode_bin[4][1024];

typedef struct {
	Uint32 xoffset, width, i, j;
} keydata;

static keydata keys[5][14] = {
	{ {0,0,0,0}, {0,0,0,0}, {23,7,7,5}, {31,7,7,5}, {39,7,7,5}, {47,7,7,5}, {55,7,7,5}, {63,7,7,5}, {71,7,7,5}, {79,7,7,5}, {255,0,0,0}, {255,0,0,0}, {255,0,0,0}, {255,0,0,0} },
	{ {3,11,7,6}, {15,7,2,5}, {23,7,0,5}, {31,7,3,2}, {39,7,6,4}, {47,7,2,2}, {55,7,6,3}, {63,7,5,3}, {71,7,4,1}, {79,7,6,1}, {87,7,7,1}, {95,11,7,6}, {255,0,0,0}, {255,0,0,0} },
	{ {3,7,4,5}, {11,7,1,2}, {19,7,3,4}, {27,7,4,2}, {35,7,6,2}, {43,7,7,2}, {51,7,0,3}, {59,7,2,3}, {67,7,3,3}, {75,7,4,3}, {83,7,3,1}, {91,15,0,6}, {107,7,1,6}, {255,0,0,0} },
	{ {1,7,3,5}, {9,7,1,4}, {17,7,7,4}, {25,7,5,2}, {33,7,2,4}, {41,7,4,4}, {49,7,1,5}, {57,7,5,4}, {65,7,1,3}, {73,7,7,3}, {81,7,0,4}, {89,7,0,2}, {97,7,5,5}, {105,7,6,5} },
	{ {0,0,0,0}, {5,7,1,0}, {13,7,2,0}, {21,7,3,0}, {29,7,4,0}, {37,7,5,0}, {45,7,6,0}, {53,7,7,0}, {61,7,1,1}, {69,7,0,1}, {77,7,0,0}, {85,7,2,1}, {93,7,5,1}, {101,7,2,6} }
};

static Uint32 keyx = 1, keyy = 4;
static keydata *current;
static Uint32 oldkey = 0xffffffff;
static Uint32 keyboard_mode = 0;
static Uint32 displayed_keyboard = 0;

static void highlight_key(void);

void keyboard_init(void) {
	/* Register PIA devices */
	PIA_0A.wrfunc[0] = keyboard_row_write;
	PIA_0B.wrfunc[0] = keyboard_column_write;
	video_blit_bothbufs(KEYBOARD_OFFSET, (PIXEL *)kbd_bin[0], 115, 44);
	keyboard_mode = 0;
	video_blit_bothbufs(1928, cmode_bin[keyboard_mode], 32, 32);
	displayed_keyboard = 0;
	keyx = 1; keyy = 4;
	current = &keys[keyy][keyx];
	highlight_key();
}

static void highlight_key(void) {
	int i;
	PIXEL *p = VIDEO_CURRENTBASE + KEYBOARD_OFFSET + current->xoffset*240
		+ 4 + keyy * 8;
	for (i = 0; i < current->width; i++) {
		*(p++) ^= ~0; *(p++) ^= ~0; *(p++) ^= ~0; *(p++) ^= ~0;
		*(p++) ^= ~0; *(p++) ^= ~0; *(p++) ^= ~0;
		p += 233;
	}
}

void keyboard_poll(void) {
	Uint32 newkeyx = keyx, newkeyy = keyy;
	Uint32 key = GpKeyGet();
	Uint32 newkey = key & oldkey;
	oldkey = ~key;
	if (newkey & GPC_VK_FL) {
		keyboard_mode++;
		if (keyboard_mode > 3)
			keyboard_mode = 0;
		video_blit_bothbufs(1928, (PIXEL *)cmode_bin[keyboard_mode], 32, 32);
	}
	if ((key & (GPC_VK_FR|GPC_VK_FL)) == (GPC_VK_FR|GPC_VK_FL))
		snapshot_state = 3;  /* reset machine */
	if (!snapshot_state && newkey & GPC_VK_START)
		snapshot_state = 2;
	switch (keyboard_mode) {
		case 3:
			joystick_rightx = joystick_righty = 255;
			joystick_leftx = joystick_lefty = 129;
			if (key & GPC_VK_LEFT) joystick_leftx = 0;
			if (key & GPC_VK_RIGHT) joystick_leftx = 255;
			if (key & GPC_VK_UP) joystick_lefty = 0;
			if (key & GPC_VK_DOWN) joystick_lefty = 255;
			if (key & GPC_VK_FB)
				PIA_TIE_LOW(PIA_0A,2);
			else
				PIA_UNTIE_LOW(PIA_0A,2);
			matrix[0][6] = key & GPC_VK_FR;
			matrix[7][5] = key & GPC_VK_FA;
			matrix[0][4] = key & GPC_VK_SELECT;
			break;
		case 2:
			joystick_leftx = joystick_lefty = 255;
			joystick_rightx = joystick_righty = 129;
			if (key & GPC_VK_LEFT) joystick_rightx = 0;
			if (key & GPC_VK_RIGHT) joystick_rightx = 255;
			if (key & GPC_VK_UP) joystick_righty = 0;
			if (key & GPC_VK_DOWN) joystick_righty = 255;
			if (key & GPC_VK_FB)
				PIA_TIE_LOW(PIA_0A,1);
			else
				PIA_UNTIE_LOW(PIA_0A,1);
			matrix[0][6] = key & GPC_VK_FR;
			matrix[7][5] = key & GPC_VK_FA;
			matrix[0][4] = key & GPC_VK_SELECT;
			break;
		case 1:
			matrix[0][6] = key & GPC_VK_FR;
			matrix[7][6] = key & GPC_VK_FB;
			matrix[3][5] = key & GPC_VK_UP;
			matrix[4][5] = key & GPC_VK_DOWN;
			matrix[5][5] = key & GPC_VK_LEFT;
			matrix[6][5] = key & GPC_VK_RIGHT;
			matrix[7][5] = key & GPC_VK_FA;
			matrix[4][2] = key & GPC_VK_SELECT;
			break;
		default:
		case 0:
			matrix[7][6] = key & GPC_VK_FR;
			if (((key&GPC_VK_FR)!=0)^displayed_keyboard) {
				displayed_keyboard ^= 1;
				video_blit_bothbufs(KEYBOARD_OFFSET, (PIXEL *)kbd_bin[displayed_keyboard], 115, 44);
				highlight_key();
			}
			/*
			if (!(key & GPC_VK_FR) && displayed_keyboard) {
				displayed_keyboard ^= 1;
				video_blit_bothbufs(KEYBOARD_OFFSET, (PIXEL *)kbd_bin[displayed_keyboard], 115, 44);
				highlight_key();
			}
			*/
			if (key & GPC_VK_UP && keyy < 4)
				newkeyy = keyy + 1;
			if (key & GPC_VK_DOWN && keyy > 0)
				newkeyy = keyy - 1;
			if (key & GPC_VK_LEFT && keyx > 0)
				newkeyx = keyx - 1;
			if (key & GPC_VK_RIGHT && keyx < 13)
				newkeyx = keyx + 1;
			while (keys[newkeyy][newkeyx].xoffset == 0)
				newkeyx++;
			while (keys[newkeyy][newkeyx].xoffset == 255)
				newkeyx--;
			if (newkeyx != keyx || newkeyy != keyy) {
				Uint32 ticks = GpTickCountGet();
				highlight_key();
				matrix[current->i][current->j] = 0;
				keyx = newkeyx; keyy = newkeyy;
				current = &keys[keyy][keyx];
				highlight_key();
				while (GpTickCountGet()-ticks<400);
			}
			matrix[current->i][current->j] = key & GPC_VK_FB;
			break;
	}
}
