/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003  Ciaran Anscomb
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "gpdef.h"
#include "gpstdlib.h"
#include "gpgraphic.h"
#include "gpstdio.h"
#include "gpfont.h"
#include "gpmm.h"
#include "gp32.h"
#include "../video.h"

//#define SOUND_SAMPLE_RATE 43457
#define SOUND_SAMPLE_RATE 21728
#define SOUND_CHANNELS 1
#define SOUND_FRAGMENT_SIZE 512
#define SOUND_BUFFER_SIZE ((SOUND_FRAGMENT_SIZE + (SOUND_FRAGMENT_SIZE / 2)))
#define SOUND_SUBCYCLES_PER_SAMPLE (((M6809_CPU_RATE*256)) / SOUND_SAMPLE_RATE)
#define SOUND_SUBCYCLES_PER_FRAGMENT (((M6809_CPU_RATE * SOUND_FRAGMENT_SIZE * 256)) / SOUND_SAMPLE_RATE)

static Uint8 fragbuf[SOUND_FRAGMENT_SIZE];
static Uint8 buffer[SOUND_BUFFER_SIZE];
static Uint8 *wrptr;
static Uint8 lastsample;
static Uint32 start_subcycle;
static volatile int idx_buf;
static volatile Uint32 playpos;

int sound_init(void) {
	GpPcmInit(PCM_M22, PCM_8BIT);
	PIA_1A.wrfunc[0] = sound_write_byte;
	wrptr = buffer;
	start_subcycle = 0;
	lastsample = 0x80;
	memset(fragbuf, 0x80, SOUND_FRAGMENT_SIZE);
	GpPcmPlay((Uint16 *)fragbuf, SOUND_FRAGMENT_SIZE, 1);
	GpPcmLock((Uint16 *)fragbuf, (int *)&idx_buf, (Uint32 *)&playpos);
	return 1;
}

void sound_reset(void) {
}

void sound_write_byte(Uint8 octet) {
	Uint8 *new_wrptr = buffer + (((current_cycle<<8) - start_subcycle)/(Uint32)SOUND_SUBCYCLES_PER_SAMPLE);
	if (!(PIA_1B.control_register & 0x08) || PIA_0B.control_register & 0x08 || PIA_0A.control_register & 0x08)
		octet = lastsample; // 0x80;
	if (new_wrptr < buffer || new_wrptr >= buffer + SOUND_BUFFER_SIZE) {
		start_subcycle = current_cycle << 8;
		wrptr = buffer;
		lastsample = octet;
		return;
	}
	for ( ; wrptr < new_wrptr; )
		*(wrptr++) = lastsample;
	if (wrptr >= buffer + SOUND_FRAGMENT_SIZE) {
		while(playpos - (Uint32)fragbuf < SOUND_FRAGMENT_SIZE);
		memcpy(fragbuf, buffer, SOUND_FRAGMENT_SIZE);
		wrptr -= SOUND_FRAGMENT_SIZE;
		start_subcycle -= SOUND_SUBCYCLES_PER_FRAGMENT;
		memcpy(buffer, buffer + SOUND_FRAGMENT_SIZE, (SOUND_BUFFER_SIZE - SOUND_FRAGMENT_SIZE));
	}
	lastsample = octet;
}
